@@ -28,6 +28,8 @@ int proxyPort = 8123;
 AtomPtr authRealm = NULL;
 AtomPtr authCredentials = NULL;
 
+AtomPtr parentAuthCredentials = NULL;
+
 AtomListPtr allowedClients = NULL;
 NetAddressPtr allowedNets = NULL;
 
@@ -66,6 +68,8 @@ preinitHttp()
                     "Authentication realm.");
     CONFIG_VARIABLE(authCredentials, CONFIG_ATOM,
                     "username:password.");
+    CONFIG_VARIABLE(parentAuthCredentials, CONFIG_ATOM,
+                    "username:password.");
     CONFIG_VARIABLE(allowedClients, CONFIG_ATOM_LIST_LOWER,
                     "Networks from which clients are allowed to connect.");
     CONFIG_VARIABLE(tunnelAllowedPorts, CONFIG_INT_LIST,
@@ -552,6 +556,7 @@ httpMakeRequest()
     request->via = NULL;
     request->persistent = 0;
     request->wait_continue = 0;
+    request->can_mutate = NULL;
     request->ohandler = NULL;
     request->requested = 0;
     request->force_error = 0;
@@ -649,14 +654,16 @@ httpCondition(ObjectPtr object, HTTPConditionPtr condition)
     if(!condition) return CONDITION_MATCH;
 
     if(condition->ims >= 0) {
-        if(condition->ims < object->last_modified)
+        if(object->last_modified < 0 ||
+           condition->ims < object->last_modified)
             return rc;
         else
             rc = CONDITION_NOT_MODIFIED;
     }
 
     if(condition->inms >= 0) {
-        if(condition->inms >= object->last_modified)
+        if(object->last_modified < 0 || 
+           condition->inms >= object->last_modified)
             return rc;
         else
             rc = CONDITION_FAILED;
@@ -684,11 +691,18 @@ httpWriteErrorHeaders(char *buf, int size, int offset, int do_body,
                       int code, AtomPtr message, int close, AtomPtr headers,
                       char *url, int url_len, char *etag)
 {
-    int n, m;
+    int n, m, i;
     char *body;
+    char htmlMessage[100];
 
     assert(code != 0);
 
+    i = htmlString(htmlMessage, 0, 100, message->string, message->length);
+    if(i < 0)
+        strcpy(htmlMessage, "(Couldn't format message)");
+    else
+        htmlMessage[MIN(i, 99)] = '\0';
+
     if(code != 304) {
         body = get_chunk();
         if(!body) {
@@ -704,7 +718,7 @@ httpWriteErrorHeaders(char *buf, int size, int offset, int do_body,
                       "\n</head><body>"
                       "\n<p>The proxy on %s:%d "
                       "encountered the following error",
-                      code, atomString(message), 
+                      code, htmlMessage, 
                       proxyName->string, proxyPort);
         if(url_len > 0) {
             m = snnprintf(body, m, CHUNK_SIZE,
@@ -717,7 +731,7 @@ httpWriteErrorHeaders(char *buf, int size, int offset, int do_body,
                       ":<br>"
                       "\n<strong>%3d %s</strong></p>"
                       "\n</body></html>\r\n",
-                      code, atomString(message));
+                      code, htmlMessage);
         if(m <= 0 || m >= CHUNK_SIZE) {
             do_log(L_ERROR, "Couldn't write error body.\n");
             dispose_chunk(body);
@@ -773,5 +787,3 @@ httpWriteErrorHeaders(char *buf, int size, int offset, int do_body,
 
     return n;
 }
-
-
