@@ -27,6 +27,7 @@ int smallRequestTime = 10;
 int replyUnpipelineTime = 20;
 int replyUnpipelineSize = 1024 * 1024;
 int pipelineAdditionalRequests = 1;
+int maxPipelineTrain = 10;
 AtomPtr parentHost = NULL;
 int parentPort = 8123;
 int pmmFirstSize = 0, pmmSize = 0;
@@ -51,6 +52,8 @@ preinitServer(void)
                     "Size for a pipeline break.");
     CONFIG_VARIABLE(pipelineAdditionalRequests, CONFIG_TRISTATE,
                     "Pipeline requests on an active connection.");
+    CONFIG_VARIABLE(maxPipelineTrain, CONFIG_INT,
+                    "Maximum number of requests pipelined at a time.");
     CONFIG_VARIABLE(pmmFirstSize, CONFIG_INT,
                     "The size of the first PMM chunk.");
     CONFIG_VARIABLE(pmmSize, CONFIG_INT,
@@ -680,7 +683,7 @@ httpServerTrigger(HTTPServerPtr server)
            progress on this server, and we don't pipeline anything
            until it succeeds.  When server->pipeline >= 4, pipelining
            is believed to work on this server. */
-        if(server->version != HTTP_11 || server->pipeline == 0 ||
+        if(server->version != HTTP_11 || server->pipeline <= 0 ||
            server->pipeline == 2 || server->pipeline == 3) {
             if(connection->pipelined == 0)
                 n = 1;
@@ -688,11 +691,11 @@ httpServerTrigger(HTTPServerPtr server)
                 n = 0;
         } else if(server->pipeline == 1) {
             if(connection->pipelined == 0)
-                n = 2;
+                n = MIN(2, maxPipelineTrain);
             else
                 n = 0;
         } else {
-            n = 10;
+            n = maxPipelineTrain;
         }
     
         idle = !connection->pipelined;
@@ -2049,6 +2052,13 @@ httpServerHandlerHeaders(int eof,
     }
 
     if(new_object != old_object) {
+        if(new_object->flags & OBJECT_INPROGRESS) {
+            /* Make sure we don't fetch this object two times at the
+               same time.  Just drop the connection. */
+            releaseObject(new_object);
+            httpServerFinish(connection, 1, 0);
+            return 1;
+        }
         old_object->flags &= ~OBJECT_VALIDATING;
         new_object->flags |= OBJECT_INPROGRESS;
         /* Signal the client side to switch to the new object -- see
@@ -2060,11 +2070,13 @@ httpServerHandlerHeaders(int eof,
         request->can_mutate = NULL;
         new_object->flags &= ~OBJECT_MUTATING;
         old_object->flags &= ~OBJECT_INPROGRESS;
-        if(request->object != new_object) {
+        if(request->object == old_object) {
             if(request->request)
                 request->request->request = NULL;
             request->request = NULL;
             request->object = new_object;
+        } else {
+            assert(request->object == new_object);
         }
         releaseNotifyObject(old_object);
         old_object = NULL;
@@ -2195,6 +2207,7 @@ httpServerIndirectHandlerCommon(HTTPConnectionPtr connection, int eof)
     HTTPRequestPtr request = connection->request;
 
     assert(eof >= 0);
+    assert(request->object->flags & OBJECT_INPROGRESS);
 
     if(connection->len > 0) {
         int rc;
@@ -2282,6 +2295,8 @@ httpServerReadData(HTTPConnectionPtr connection, int immediate)
     ObjectPtr object = request->object;
     int to = -1;
 
+    assert(object->flags & OBJECT_INPROGRESS);
+
     if(request->request == NULL) {
         httpServerFinish(connection, 1, 0);
         return 1;
